@@ -23,7 +23,7 @@
 
 #include <algorithm>
 
-#include "procman.h"
+#include "procman-app.h"
 #include "load-graph.h"
 #include "util.h"
 #include "gsm_color_button.h"
@@ -76,6 +76,7 @@ void draw_background(LoadGraph *graph) {
     PangoLayout* layout;
     PangoFontDescription* font_desc;
     PangoRectangle extents;
+    GdkRGBA fg, bg;
 
     num_bars = graph->num_bars();
     graph->graph_dely = (graph->draw_height - 15) / num_bars; /* round to int to avoid AA blur */
@@ -90,13 +91,16 @@ void draw_background(LoadGraph *graph) {
                                                            allocation.height);
     cr = cairo_create (graph->background);
 
+    GtkStyleContext *context = gtk_widget_get_style_context (ProcmanApp::get()->notebook);
+    gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
+    gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &fg);
+
     // set the background colour
-    GtkStyle *style = gtk_widget_get_style (ProcData::get_instance()->notebook);
-    gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_NORMAL]);
+    gdk_cairo_set_source_rgba (cr, &bg);
     cairo_paint (cr);
 
     layout = pango_cairo_create_layout (cr);
-    font_desc = pango_font_description_copy (style->font_desc);
+    font_desc = pango_font_description_copy (gtk_style_context_get_font (context, GTK_STATE_FLAG_NORMAL));
     pango_font_description_set_size (font_desc, 0.8 * graph->fontsize * PANGO_SCALE);
     pango_layout_set_font_description (layout, font_desc);
     pango_font_description_free (font_desc);
@@ -123,7 +127,7 @@ void draw_background(LoadGraph *graph) {
         else
             y = i * graph->graph_dely + graph->fontsize / 2.0;
 
-        gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_NORMAL]);
+        gdk_cairo_set_source_rgba (cr, &fg);
         if (graph->type == LOAD_GRAPH_NET) {
             // operation orders matters so it's 0 if i == num_bars
             guint64 rate = graph->net.max - (i * graph->net.max / num_bars);
@@ -135,7 +139,11 @@ void draw_background(LoadGraph *graph) {
             pango_cairo_show_layout (cr, layout);
         } else {
             // operation orders matters so it's 0 if i == num_bars
-            caption = g_strdup_printf("%d%%", 100 - i * (100 / num_bars));
+            guint max = 100;
+            if (graph->type == LOAD_GRAPH_CPU && !ProcmanApp::get()->config.solaris_mode) {
+                max = 100 * graph->n;
+            }
+            caption = g_strdup_printf("%d %%", max - i * (max / num_bars));
             pango_layout_set_text (layout, caption, -1);
             pango_layout_get_extents (layout, NULL, &extents);
             cairo_move_to (cr, graph->indent - 1.0 * extents.width / PANGO_SCALE + 20,
@@ -172,7 +180,7 @@ void draw_background(LoadGraph *graph) {
         cairo_move_to (cr,
                        (ceil(x) + 0.5 + graph->rmargin + graph->indent) - (1.0 * extents.width / PANGO_SCALE / 2),
                        graph->draw_height - 1.0 * extents.height / PANGO_SCALE);
-        gdk_cairo_set_source_color (cr, &style->fg[GTK_STATE_NORMAL]);
+        gdk_cairo_set_source_rgba (cr, &fg);
         pango_cairo_show_layout (cr, layout);
         g_free (caption);
     }
@@ -218,7 +226,8 @@ load_graph_draw (GtkWidget *widget,
     LoadGraph * const graph = static_cast<LoadGraph*>(data_ptr);
     GdkWindow *window;
 
-    guint i, j;
+    guint i;
+    gint j;
     gdouble sample_width, x_offset;
 
     window = gtk_widget_get_window (graph->disp);
@@ -251,10 +260,14 @@ load_graph_draw (GtkWidget *widget,
                      graph->real_draw_height + FRAME_WIDTH - 1);
     cairo_clip(cr);
 
-    for (j = 0; j < graph->n; ++j) {
-        cairo_move_to (cr, x_offset, (1.0f - graph->data[0][j]) * graph->real_draw_height);
-        gdk_cairo_set_source_color (cr, &(graph->colors [j]));
-
+    bool drawStacked = graph->type == LOAD_GRAPH_CPU && ProcmanApp::get()->config.draw_stacked;
+    for (j = graph->n-1; j >= 0; j--) {
+        gdk_cairo_set_source_rgba (cr, &(graph->colors [j]));
+        if (drawStacked) {
+            cairo_move_to (cr, x_offset, graph->real_draw_height);
+        } else {
+            cairo_move_to (cr, x_offset, (1.0f - graph->data[0][j]) * graph->real_draw_height);
+        }
         for (i = 1; i < LoadGraph::NUM_POINTS; ++i) {
             if (graph->data[i][j] == -1.0f)
                 continue;
@@ -266,8 +279,12 @@ load_graph_draw (GtkWidget *widget,
                             x_offset - (i * graph->graph_delx),
                             (1.0f - graph->data[i][j]) * graph->real_draw_height + 3.5f);
         }
-        cairo_stroke (cr);
-
+        if (drawStacked) {
+            cairo_rel_line_to (cr, x_offset - (LoadGraph::NUM_POINTS * graph->graph_delx), graph->real_draw_height);
+            cairo_fill(cr);
+        } else {
+            cairo_stroke (cr);
+        }
     }
 
     cairo_destroy (cr);
@@ -314,7 +331,10 @@ get_load (LoadGraph *graph)
         used  = NOW[i][CPU_USED]  - LAST[i][CPU_USED];
 
         load = used / MAX(total, 1.0f);
-        graph->data[0][i] = load;
+        graph->data[0][i] = load / graph->n;
+        if (i > 0) {
+            graph->data[0][i] += graph->data[0][i-1];
+        }
 
         /* Update label */
         text = g_strdup_printf("%.1f%%", load * 100.0f);
@@ -443,7 +463,7 @@ net_scale (LoadGraph *graph, guint64 din, guint64 dout)
 
     const guint64 bak_max(new_max);
 
-    if (ProcData::get_instance()->config.network_in_bits) {
+    if (ProcmanApp::get()->config.network_in_bits) {
         // nice number is for the ticks
         unsigned ticks = graph->num_bars();
 
@@ -686,7 +706,7 @@ LoadGraph::LoadGraph(guint type)
     switch (type) {
         case LOAD_GRAPH_CPU:
             memset(&cpu, 0, sizeof cpu);
-            n = ProcData::get_instance()->config.num_cpus;
+            n = ProcmanApp::get()->config.num_cpus;
 
             for(guint i = 0; i < G_N_ELEMENTS(labels.cpu); ++i)
                 labels.cpu[i] = gtk_label_new(NULL);
@@ -720,26 +740,26 @@ LoadGraph::LoadGraph(guint type)
             break;
     }
 
-    speed  = ProcData::get_instance()->config.graph_update_interval;
+    speed  = ProcmanApp::get()->config.graph_update_interval;
 
     colors.resize(n);
 
     switch (type) {
         case LOAD_GRAPH_CPU:
-            memcpy(&colors[0], ProcData::get_instance()->config.cpu_color,
+            memcpy(&colors[0], ProcmanApp::get()->config.cpu_color,
                    n * sizeof colors[0]);
             break;
         case LOAD_GRAPH_MEM:
-            colors[0] = ProcData::get_instance()->config.mem_color;
-            colors[1] = ProcData::get_instance()->config.swap_color;
+            colors[0] = ProcmanApp::get()->config.mem_color;
+            colors[1] = ProcmanApp::get()->config.swap_color;
             mem_color_picker = gsm_color_button_new (&colors[0],
                                                         GSMCP_TYPE_PIE);
             swap_color_picker = gsm_color_button_new (&colors[1],
                                                          GSMCP_TYPE_PIE);
             break;
         case LOAD_GRAPH_NET:
-            colors[0] = ProcData::get_instance()->config.net_in_color;
-            colors[1] = ProcData::get_instance()->config.net_out_color;
+            colors[0] = ProcmanApp::get()->config.net_in_color;
+            colors[1] = ProcmanApp::get()->config.net_out_color;
             break;
     }
 
